aboutsummaryrefslogtreecommitdiff
path: root/odb/sqlite/query.hxx
diff options
context:
space:
mode:
Diffstat (limited to 'odb/sqlite/query.hxx')
-rw-r--r--odb/sqlite/query.hxx62
1 files changed, 39 insertions, 23 deletions
diff --git a/odb/sqlite/query.hxx b/odb/sqlite/query.hxx
index 54e0b2b..ee05feb 100644
--- a/odb/sqlite/query.hxx
+++ b/odb/sqlite/query.hxx
@@ -20,7 +20,9 @@
#include <odb/sqlite/traits.hxx>
#include <odb/sqlite/sqlite-types.hxx>
#include <odb/sqlite/binding.hxx>
+
#include <odb/sqlite/details/export.hxx>
+#include <odb/sqlite/details/conversion.hxx>
namespace odb
{
@@ -129,7 +131,7 @@ namespace odb
clause_part (bool p): kind (boolean), bool_part (p) {}
kind_type kind;
- std::string part;
+ std::string part; // If kind is param, then part is conversion expr.
bool bool_part;
};
@@ -172,7 +174,8 @@ namespace odb
query (val_bind<T> v)
: parameters_ (new (details::shared) query_params)
{
- append<T, type_traits<T>::db_type_id> (v);
+ append<T, type_traits<T>::db_type_id> (
+ v, details::conversion<T>::to ());
}
template <typename T>
@@ -180,7 +183,8 @@ namespace odb
query (ref_bind<T> r)
: parameters_ (new (details::shared) query_params)
{
- append<T, type_traits<T>::db_type_id> (r);
+ append<T, type_traits<T>::db_type_id> (
+ r, details::conversion<T>::to ());
}
template <database_type_id ID>
@@ -254,7 +258,8 @@ namespace odb
query&
operator+= (val_bind<T> v)
{
- append<T, type_traits<T>::db_type_id> (v);
+ append<T, type_traits<T>::db_type_id> (
+ v, details::conversion<T>::to ());
return *this;
}
@@ -262,18 +267,19 @@ namespace odb
query&
operator+= (ref_bind<T> r)
{
- append<T, type_traits<T>::db_type_id> (r);
+ append<T, type_traits<T>::db_type_id> (
+ r, details::conversion<T>::to ());
return *this;
}
public:
template <typename T, database_type_id ID>
void
- append (val_bind<T>);
+ append (val_bind<T>, const char* conv);
template <typename T, database_type_id ID>
void
- append (ref_bind<T>);
+ append (ref_bind<T>, const char* conv);
void
append (const std::string& native);
@@ -283,7 +289,7 @@ namespace odb
private:
void
- add (details::shared_ptr<query_param>);
+ add (details::shared_ptr<query_param>, const char* conv);
private:
typedef std::vector<clause_part> clause_type;
@@ -421,10 +427,11 @@ namespace odb
template <typename T, database_type_id ID>
struct query_column
{
// Note that we keep shalow copies of the table and column names.
//
- query_column (const char* table, const char* column)
- : table_ (table), column_ (column)
+ query_column (const char* table, const char* column, const char* conv)
+ : table_ (table), column_ (column), conversion_ (conv)
{
}
@@ -440,6 +447,14 @@ namespace odb
return column_;
}
+ // Can be NULL.
+ //
+ const char*
+ conversion () const
+ {
+ return conversion_;
+ }
+
// is_null, is_not_null
//
public:
@@ -492,7 +507,7 @@ namespace odb
{
query q (table_, column_);
q += "=";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -509,7 +524,7 @@ namespace odb
{
query q (table_, column_);
q += "=";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -577,7 +592,7 @@ namespace odb
{
query q (table_, column_);
q += "!=";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -594,7 +609,7 @@ namespace odb
{
query q (table_, column_);
q += "!=";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -662,7 +677,7 @@ namespace odb
{
query q (table_, column_);
q += "<";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -679,7 +694,7 @@ namespace odb
{
query q (table_, column_);
q += "<";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -747,7 +762,7 @@ namespace odb
{
query q (table_, column_);
q += ">";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -764,7 +779,7 @@ namespace odb
{
query q (table_, column_);
q += ">";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -832,7 +847,7 @@ namespace odb
{
query q (table_, column_);
q += "<=";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -849,7 +864,7 @@ namespace odb
{
query q (table_, column_);
q += "<=";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -917,7 +932,7 @@ namespace odb
{
query q (table_, column_);
q += ">=";
- q.append<T, ID> (v);
+ q.append<T, ID> (v, conversion_);
return q;
}
@@ -934,7 +949,7 @@ namespace odb
{
query q (table_, column_);
q += ">=";
- q.append<T, ID> (r);
+ q.append<T, ID> (r, conversion_);
return q;
}
@@ -1078,6 +1093,7 @@ namespace odb
private:
const char* table_;
const char* column_;
+ const char* conversion_;
};
// Provide operator+() for using columns to construct native
@@ -421,10 +427,11 @@ namespace odb
struct query_column
{
- // Note that we keep shalow copies of the table and column names.
+ // Note that we keep shallow copies of the table, column, and conversion
+ // expression. The latter can be NULL.
//